unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, TeeProcs, TeEngine, TeeTools, TeePolar, Chart, Series, StdCtrls;

const
  CHART_POLAR = 0;
  CHART_LINE = 1;
  CHART_BAR = 2;
  CHART_POINT = 3;
  LABEL_LIST : array[0..12] of string = ('Record','Date','Time','Ia','Ib','Ic','Ig','In','Va','Vb','Vc','Phase','Status');
type
  TArrayOfArrayOfDouble = array of array of double;
  TArrayOfDouble = array of double;
  TForm1 = class(TForm)
    Chart1: TChart;
    Button1: TButton;
    procedure FormCreate(Sender : TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    ChartDisplay            : TChart;
    ChartSeries             : array of TCustomSeries;
    BarChartSeries          : array of TCustomBarSeries;
    PolarChartSeries        : array of TCustomPolarSeries;
//    BarSeries               : TBarSeries;        // used for Harmonics
//    PointSeries             : TPointSeries;  // used for LDP events

    VariableCount           : integer;
    ValueCount              : integer;
    ChartType               : integer;
    ShowLabels              : boolean;

    xArray : TArrayOfDouble;
    yArray : TArrayOfArrayOfDouble;

    procedure InitChartAttributes();
    procedure AllocateChartArrays();
    procedure InitChartArrays();
    procedure InitPolarArrays();
    procedure FillChartArrays();
    procedure FreeCharts();
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
procedure TForm1.Button1Click(Sender: TObject);
begin
//
  ChartType := ChartType + 1;
  if(ChartType > 2) then
  begin
    ChartType := 0;
  end;
  FreeCharts();
  AllocateChartArrays();
  InitChartAttributes();
  InitChartArrays();
  if(ChartType = CHART_POLAR) then
  begin
    InitPolarArrays();
  end else
  begin
    FillChartArrays();
  end;
  ChartDisplay.Update();  // refresh display

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
//
  ShowLabels := false;
  VariableCount := 10;
  ChartType := 0;
  ValueCount := 100;
  ChartDisplay := Chart1;
  ChartDisplay.View3D := false;
end;

  procedure TForm1.AllocateChartArrays();
var
  index : integer;
begin
  // Make sure old chart data is freed before reallocating arrays
  FreeCharts();

  if(ChartType = CHART_POLAR) then
  begin
    SetLength(PolarChartSeries,VariableCount div 2);
    SetLength(ChartSeries,0);
  end else
  if(ChartType = CHART_BAR) then
  begin
    SetLength(BarChartSeries,VariableCount);
    SetLength(ChartSeries,0);
  end else
  if(ChartType = CHART_LINE) then
  begin
    SetLength(PolarChartSeries,0);
    SetLength(BarChartSeries,0);
    SetLength(ChartSeries,VariableCount);
  end;

  SetLength(xArray, ValueCount);
  SetLength(yArray, VariableCount);

  for index := 0 to VariableCount - 1 do
  begin
    SetLength(yArray[index],ValueCount);
  end;

end;

procedure TForm1.InitChartAttributes();
const
  CBEMA_TITLES : array [0..1] of string = ('ITI Sag','ITI Swell');
  EVENT_TITLE = 'Event';
var
  j : integer;
begin

  // Allocate and initialize chart series

  for j := 0 to VariableCount-1 do        // for each channel
  begin
    if(ChartType = CHART_POLAR) then
    begin
      if(j < VariableCount div 2) then
      begin
        PolarChartSeries[j] := TPolarBarSeries.Create(self);    // create polar chart
        PolarChartSeries[j].Title := LABEL_LIST[j];  // assign channel name to legend
        PolarChartSeries[j].Marks.Visible := ShowLabels;
        PolarChartSeries[j].CircleLabels := true;
        PolarChartSeries[j].ClockWiseLabels := false;
        ChartDisplay.AddSeries(PolarChartSeries[j]);           // add the new channel series to the chart
      end;
    end else
    if(ChartType = CHART_BAR) then
    begin
      BarChartSeries[j] := TBarSeries.Create(self);    // create bar chart
      BarChartSeries[j].Title := LABEL_LIST[j];  // assign channel name to legend
      BarChartSeries[j].Marks.Visible := ShowLabels;
      ChartDisplay.AddSeries(BarChartSeries[j]);           // add the new channel series to the chart
    end else
    if(ChartType = CHART_LINE) then
    begin
      ChartSeries[j] := TLineSeries.Create(self);    // create line chart
      ChartSeries[j].Title := LABEL_LIST[j];  // assign channel name to legend
      ChartDisplay.AddSeries(ChartSeries[j]);           // add the new channel series to the chart
    end;
  end;
end;

procedure TForm1.InitChartArrays();
var
  index : integer;
  indep : integer;
begin
  for index := 0 to ValueCount -1 do
  begin
    xArray[index] := index;
    for indep := 0 to VariableCount -1 do
    begin
      yArray[indep][index] := 2 * (indep + 1) * (index + 1);
    end;
  end;
end;

procedure TForm1.InitPolarArrays();
var
  j : integer;
  mag, angle : array of Double;
begin
  SetLength(mag,VariableCount div 2);
  SetLength(angle,VariableCount div 2);

  for j := 0 to VariableCount-1 do     // add every channel for this record to the chart
  begin
    if((j mod 2) = 0) then    // magnitude
    begin
      mag[j div 2] := yArray[0][j];
      angle[j div 2] := yArray[0][j+1];
    end;
  end;

  for j := 0 to VariableCount div 2-1 do     // add every channel for this record to the chart
  begin
    ChartDisplay.Series[j].AddXY(angle[j],mag[j]);
  end;
end;

procedure TForm1.FillChartArrays();
var
  index : integer;
begin
  // Send data to chart for each channel if using dynamic arrays for speed
  for index := 0 to VariableCount - 1 do
  begin
    With ChartDisplay.Series[index].XValues do
    begin
      Value := TChartValues(xArray);
      Count := ValueCount;
      Modified := true;
    end;
    With ChartDisplay.Series[index].YValues do
    begin
      Value := TChartValues(yArray[index]);
      Count := ValueCount;
      Modified := true;
    end;
  end;
end;

procedure TForm1.FreeCharts();
var
  count : integer;
begin
    try
      // There is a chronic problem related to freeing series in TeeChart.
      // This try/catch block is intended to minimize the impact of this problem.
      if(Assigned(ChartDisplay)) then
      begin
        count := ChartDisplay.SeriesCount();
        if(count > 0) then
        begin
          ChartDisplay.FreeAllSeries();   // throw away previous chart data
        end;
      end;
    finally
      count := 0;
      // nothing to do
    end;
end;


end.
